@@ -6,12 +6,22 @@ from .dict import IntervalDict
 
 
 __all__ = [
-    'inf', 'CLOSED', 'OPEN',
-    'Interval',
-    'open', 'closed', 'openclosed', 'closedopen', 'singleton', 'empty',
-    'iterate',
-    'from_string', 'to_string', 'from_data', 'to_data',
-    'IntervalDict',
+    "inf",
+    "CLOSED",
+    "OPEN",
+    "Interval",
+    "open",
+    "closed",
+    "openclosed",
+    "closedopen",
+    "singleton",
+    "empty",
+    "iterate",
+    "from_string",
+    "to_string",
+    "from_data",
+    "to_data",
+    "IntervalDict",
 ]
 
 CLOSED = Bound.CLOSED
