@@ -1,27 +1,26 @@
+import warnings
+
 from collections import namedtuple
 from .const import Bound, inf
 
 
-Atomic = namedtuple('Atomic', ['left', 'lower', 'upper', 'right'])
+Atomic = namedtuple("Atomic", ["left", "lower", "upper", "right"])
 
 
 def mergeable(a, b):
     """
-    Tester whether two atomic intervals can be merged (i.e. they overlap or are adjacent).
+    Tester whether two atomic intervals can be merged (i.e. they overlap or
+    are adjacent).
 
     :param a: an atomic interval.
     :param b: an atomic interval.
     :return: True if mergeable, False otherwise.
     """
-    if a.lower < b.lower or (a.lower == b.lower and a.left == Bound.CLOSED):
-        first, second = a, b
-    else:
-        first, second = b, a
-
-    if first.upper == second.lower:
-        return first.right == Bound.CLOSED or second.left == Bound.CLOSED
-
-    return first.upper > second.lower
+    warnings.warn(
+        "This function is deprecated, use Interval._mergeable instead.",
+        DeprecationWarning,
+    )
+    return Interval._mergeable(a, b)
 
 
 def open(lower, upper):
@@ -92,11 +91,12 @@ class Interval:
     This class represents an interval.
 
     An interval is an (automatically simplified) union of atomic intervals.
-    It can be created with Interval.from_atomic(), by passing intervals to __init__, or by using
-    one of the helpers provided in this module (open, closed, openclosed, etc.)
+    It can be created with Interval.from_atomic(), by passing intervals to
+    __init__, or by using one of the helpers provided in this module (open,
+    closed, openclosed, etc.)
     """
 
-    __slots__ = ('_intervals',)
+    __slots__ = ("_intervals",)
 
     def __init__(self, *intervals):
         """
@@ -111,7 +111,7 @@ class Interval:
                 if not interval.empty:
                     self._intervals.extend(interval._intervals)
             else:
-                raise TypeError('Parameters must be Interval instances')
+                raise TypeError("Parameters must be Interval instances")
 
         if len(self._intervals) == 0:
             # So we have at least one (empty) interval
@@ -126,20 +126,32 @@ class Interval:
                 current = self._intervals[i]
                 successor = self._intervals[i + 1]
 
-                if mergeable(current, successor):
+                if self.__class__._mergeable(current, successor):
                     if current.lower == successor.lower:
                         lower = current.lower
-                        left = current.left if current.left == Bound.CLOSED else successor.left
+                        left = (
+                            current.left
+                            if current.left == Bound.CLOSED
+                            else successor.left
+                        )
                     else:
                         lower = min(current.lower, successor.lower)
-                        left = current.left if lower == current.lower else successor.left
+                        left = (
+                            current.left if lower == current.lower else successor.left
+                        )
 
                     if current.upper == successor.upper:
                         upper = current.upper
-                        right = current.right if current.right == Bound.CLOSED else successor.right
+                        right = (
+                            current.right
+                            if current.right == Bound.CLOSED
+                            else successor.right
+                        )
                     else:
                         upper = max(current.upper, successor.upper)
-                        right = current.right if upper == current.upper else successor.right
+                        right = (
+                            current.right if upper == current.upper else successor.right
+                        )
 
                     union = Atomic(left, lower, upper, right)
                     self._intervals.pop(i)  # pop current
@@ -148,6 +160,26 @@ class Interval:
                 else:
                     i = i + 1
 
+    @classmethod
+    def _mergeable(cls, a, b):
+        """
+        Tester whether two atomic intervals can be merged (i.e. they overlap or
+        are adjacent).
+
+        :param a: an atomic interval.
+        :param b: an atomic interval.
+        :return: True if mergeable, False otherwise.
+        """
+        if a.lower < b.lower or (a.lower == b.lower and a.left == Bound.CLOSED):
+            first, second = a, b
+        else:
+            first, second = b, a
+
+        if first.upper == second.lower:
+            return first.right == Bound.CLOSED or second.left == Bound.CLOSED
+
+        return first.upper > second.lower
+
     @property
     def left(self):
         """
@@ -181,21 +213,22 @@ class Interval:
         """
         True if interval is empty, False otherwise.
         """
-        return (
-            self.lower > self.upper or
-            (self.lower == self.upper and (self.left == Bound.OPEN or self.right == Bound.OPEN))
+        return self.lower > self.upper or (
+            self.lower == self.upper
+            and (self.left == Bound.OPEN or self.right == Bound.OPEN)
         )
 
     @property
     def atomic(self):
         """
         True if this interval is atomic, False otherwise.
-        An interval is atomic if it is composed of a single (possibly empty) atomic interval.
+        An interval is atomic if it is composed of a single (possibly empty)
+        atomic interval.
         """
         return len(self._intervals) == 1
 
-    @staticmethod
-    def from_atomic(left, lower, upper, right):
+    @classmethod
+    def from_atomic(cls, left, lower, upper, right):
         """
         Create an Interval instance containing a single atomic interval.
 
@@ -204,13 +237,13 @@ class Interval:
         :param upper: value of the upper bound.
         :param right: either CLOSED or OPEN.
         """
-        instance = Interval()
+        instance = cls()
         left = left if lower not in [inf, -inf] else Bound.OPEN
         right = right if upper not in [inf, -inf] else Bound.OPEN
 
         instance._intervals = [Atomic(left, lower, upper, right)]
         if instance.empty:
-            return Interval()
+            return cls()
 
         return instance
 
@@ -222,9 +255,11 @@ class Interval:
 
         :return: an Interval instance.
         """
-        return Interval.from_atomic(self.left, self.lower, self.upper, self.right)
+        return self.__class__.from_atomic(self.left, self.lower, self.upper, self.right)
 
-    def replace(self, left=None, lower=None, upper=None, right=None, *, ignore_inf=True):
+    def replace(
+        self, left=None, lower=None, upper=None, right=None, *, ignore_inf=True
+    ):
         """
         Create a new interval based on the current one and the provided values.
 
@@ -232,15 +267,16 @@ class Interval:
         its enclosure satisfies the new bounds. In other words, its new enclosure
         will be equal to self.enclosure.replace(left, lower, upper, right).
 
-        Callable can be passed instead of values. In that case, it is called with the current
-        corresponding value except if ignore_inf if set (default) and the corresponding
-        bound is an infinity.
+        Callable can be passed instead of values. In that case, it is called with the
+        current corresponding value except if ignore_inf if set (default) and the
+        corresponding bound is an infinity.
 
         :param left: (a function of) left boundary.
         :param lower: (a function of) value of the lower bound.
         :param upper: (a function of) value of the upper bound.
         :param right: (a function of) right boundary.
-        :param ignore_inf: ignore infinities if functions are provided (default is True).
+        :param ignore_inf: ignore infinities if functions are provided (default
+            is True).
         :return: an Interval instance
         """
         enclosure = self.enclosure
@@ -272,25 +308,25 @@ class Interval:
             right = enclosure.right if right is None else right
 
         if self.atomic:
-            return Interval.from_atomic(left, lower, upper, right)
+            return self.__class__.from_atomic(left, lower, upper, right)
 
-        n_interval = self & Interval.from_atomic(left, lower, upper, right)
+        n_interval = self & self.__class__.from_atomic(left, lower, upper, right)
 
         if n_interval.atomic:
             return n_interval.replace(left, lower, upper, right)
         else:
             lowest = n_interval[0].replace(left=left, lower=lower)
             highest = n_interval[-1].replace(upper=upper, right=right)
-            return Interval(lowest, *n_interval[1:-1], highest)
+            return self.__class__(lowest, *n_interval[1:-1], highest)
 
     def apply(self, func):
         """
-        Apply a function on each of the underlying atomic intervals and return their union
-        as a new interval instance
+        Apply a function on each of the underlying atomic intervals and return their
+        union as a new interval instance
 
-        Given function is expected to return an interval (possibly empty or not atomic) or
-        a 4-uple (left, lower, upper, right) whose values correspond to the parameters of
-        Interval.from_atomic(left, lower, upper, right).
+        Given function is expected to return an interval (possibly empty or not
+        atomic) or a 4-uple (left, lower, upper, right) whose values correspond to
+        the parameters of Interval.from_atomic(left, lower, upper, right).
 
         This method is merely a shortcut for Interval(*list(map(func, self))).
 
@@ -305,17 +341,25 @@ class Interval:
             if isinstance(value, Interval):
                 intervals.append(value)
             elif isinstance(value, tuple):
-                intervals.append(Interval.from_atomic(*value))
+                intervals.append(self.__class__.from_atomic(*value))
             else:
-                raise TypeError('Unsupported return type {} for {}'.format(type(value), value))
+                raise TypeError(
+                    "Unsupported return type {} for {}".format(type(value), value)
+                )
 
-        return Interval(*intervals)
+        return self.__class__(*intervals)
 
     def adjacent(self, other):
         """
-        Test if given interval is adjacent.
+        Test if two intervals are adjacent.
 
-        An interval is adjacent if there is no intersection, and their union is an atomic interval.
+        Two intervals are adjacent if they do not overlap and their union form a
+        single atomic interval.
+
+        While this definition corresponds to the usual notion of adjacency for atomic
+        intervals, it has stronger requirements for non-atomic ones since it requires
+        all underlying atomic intervals to be adjacent (i.e. that one
+        interval fills the gaps between the atomic intervals of the other one).
 
         :param other: an interval.
         :return: True if intervals are adjacent, False otherwise.
@@ -324,12 +368,16 @@ class Interval:
 
     def overlaps(self, other):
         """
-        Test if intervals have any overlapping value.
+        Test if two intervals overlap (i.e. if their intersection is non-empty).
 
         :param other: an interval.
         :return: True if intervals overlap, False otherwise.
         """
         if isinstance(other, Interval):
+            if self.upper < other.lower or self.lower > other.upper:
+                # Early out for clearly non-overlapping intervals
+                return False
+
             i_iter = iter(self)
             o_iter = iter(other)
             i_current = next(i_iter)
@@ -344,7 +392,7 @@ class Interval:
                     return True
             return False
         else:
-            raise TypeError('Unsupported type {} for {}'.format(type(other), other))
+            raise TypeError("Unsupported type {} for {}".format(type(other), other))
 
     def intersection(self, other):
         """
@@ -391,23 +439,41 @@ class Interval:
         """
         return self - other
 
+    def __getattr__(self, name):
+        if name == "__next__":
+            # Hack for a better representation of intervals within pandas
+            # See https://github.com/AlexandreDecan/portion/pull/54
+            try:
+                import inspect
+
+                if inspect.stack()[1].function == "pprint_thing":
+                    return
+            except Exception:
+                pass
+        raise AttributeError
+
     def __len__(self):
         return len(self._intervals)
 
     def __iter__(self):
-        return iter([Interval.from_atomic(*i) for i in self._intervals])
+        yield from (self.__class__.from_atomic(*i) for i in self._intervals)
 
     def __getitem__(self, item):
         if isinstance(item, slice):
-            return [Interval.from_atomic(*i) for i in self._intervals[item]]
+            return self.__class__(
+                *[self.__class__.from_atomic(*i) for i in self._intervals[item]]
+            )
         else:
-            return Interval.from_atomic(*self._intervals[item])
+            return self.__class__.from_atomic(*self._intervals[item])
 
     def __and__(self, other):
         if not isinstance(other, Interval):
             return NotImplemented
 
-        if self.atomic and other.atomic:
+        if self.upper < other.lower or self.lower > other.upper:
+            # Early out for non-overlapping intervals
+            return self.__class__()
+        elif self.atomic and other.atomic:
             if self.lower == other.lower:
                 lower = self.lower
                 left = self.left if self.left == Bound.OPEN else other.left
@@ -422,7 +488,7 @@ class Interval:
                 upper = min(self.upper, other.upper)
                 right = self.right if upper == self.upper else other.right
 
-            return Interval.from_atomic(left, lower, upper, right)
+            return self.__class__.from_atomic(left, lower, upper, right)
         else:
             intersections = []
 
@@ -449,24 +515,29 @@ class Interval:
                     else:
                         assert False
 
-            return Interval(*intersections)
+            return self.__class__(*intersections)
 
     def __or__(self, other):
         if isinstance(other, Interval):
-            return Interval(self, other)
+            return self.__class__(self, other)
         else:
             return NotImplemented
 
     def __contains__(self, item):
         if isinstance(item, Interval):
-            if self.atomic:
+            if item.empty:
+                return True
+            elif self.upper < item.lower or self.lower > item.upper:
+                # Early out for non-overlapping intervals
+                return False
+            elif self.atomic:
                 left = item.lower > self.lower or (
-                    item.lower == self.lower and
-                    (item.left == self.left or self.left == Bound.CLOSED)
+                    item.lower == self.lower
+                    and (item.left == self.left or self.left == Bound.CLOSED)
                 )
                 right = item.upper < self.upper or (
-                    item.upper == self.upper and
-                    (item.right == self.right or self.right == Bound.CLOSED)
+                    item.upper == self.upper
+                    and (item.right == self.right or self.right == Bound.CLOSED)
                 )
                 return left and right
             else:
@@ -486,25 +557,30 @@ class Interval:
                 return True
         else:
             # Item is a value
+            if self.upper < item or self.lower > item:
+                return False
+
             for i in self._intervals:
                 left = (item >= i.lower) if i.left == Bound.CLOSED else (item > i.lower)
-                right = (item <= i.upper) if i.right == Bound.CLOSED else (item < i.upper)
+                right = (
+                    (item <= i.upper) if i.right == Bound.CLOSED else (item < i.upper)
+                )
                 if left and right:
                     return True
             return False
 
     def __invert__(self):
         complements = [
-            Interval.from_atomic(Bound.OPEN, -inf, self.lower, ~self.left),
-            Interval.from_atomic(~self.right, self.upper, inf, Bound.OPEN)
+            self.__class__.from_atomic(Bound.OPEN, -inf, self.lower, ~self.left),
+            self.__class__.from_atomic(~self.right, self.upper, inf, Bound.OPEN),
         ]
 
         for i, j in zip(self._intervals[:-1], self._intervals[1:]):
             complements.append(
-                Interval.from_atomic(~i.right, i.upper, j.lower, ~j.left)
+                self.__class__.from_atomic(~i.right, i.upper, j.lower, ~j.left)
             )
 
-        return Interval(*complements)
+        return self.__class__(*complements)
 
     def __sub__(self, other):
         if isinstance(other, Interval):
@@ -519,10 +595,10 @@ class Interval:
 
             for a, b in zip(self._intervals, other._intervals):
                 eq = (
-                    a.left == b.left and
-                    a.lower == b.lower and
-                    a.upper == b.upper and
-                    a.right == b.right
+                    a.left == b.left
+                    and a.lower == b.lower
+                    and a.upper == b.upper
+                    and a.right == b.right
                 )
                 if not eq:
                     return False
@@ -535,59 +611,71 @@ class Interval:
             if self.right == Bound.OPEN:
                 return self.upper <= other.lower
             else:
-                return self.upper < other.lower or \
-                    (self.upper == other.lower and other.left == Bound.OPEN)
+                return self.upper < other.lower or (
+                    self.upper == other.lower and other.left == Bound.OPEN
+                )
         else:
-            return self.upper < other or (self.right == Bound.OPEN and self.upper == other)
+            return self.upper < other or (
+                self.right == Bound.OPEN and self.upper == other
+            )
 
     def __gt__(self, other):
         if isinstance(other, Interval):
             if self.left == Bound.OPEN:
                 return self.lower >= other.upper
             else:
-                return self.lower > other.upper or \
-                    (self.lower == other.upper and other.right == Bound.OPEN)
+                return self.lower > other.upper or (
+                    self.lower == other.upper and other.right == Bound.OPEN
+                )
         else:
-            return self.lower > other or (self.left == Bound.OPEN and self.lower == other)
+            return self.lower > other or (
+                self.left == Bound.OPEN and self.lower == other
+            )
 
     def __le__(self, other):
         if isinstance(other, Interval):
             if self.right == Bound.OPEN:
                 return self.upper <= other.upper
             else:
-                return self.upper < other.upper or \
-                    (self.upper == other.upper and other.right == Bound.CLOSED)
+                return self.upper < other.upper or (
+                    self.upper == other.upper and other.right == Bound.CLOSED
+                )
         else:
-            return self.lower < other or (self.left == Bound.CLOSED and self.lower == other)
+            return self.lower < other or (
+                self.left == Bound.CLOSED and self.lower == other
+            )
 
     def __ge__(self, other):
         if isinstance(other, Interval):
             if self.left == Bound.OPEN:
                 return self.lower >= other.lower
             else:
-                return self.lower > other.lower or \
-                    (self.lower == other.lower and other.left == Bound.CLOSED)
+                return self.lower > other.lower or (
+                    self.lower == other.lower and other.left == Bound.CLOSED
+                )
         else:
-            return self.upper > other or (self.right == Bound.CLOSED and self.upper == other)
+            return self.upper > other or (
+                self.right == Bound.CLOSED and self.upper == other
+            )
 
     def __hash__(self):
         return hash(tuple([self.lower, self.upper]))
 
     def __repr__(self):
-        intervals = []
+        if self.empty:
+            return "()"
 
-        for interval in self:
-            if interval.empty:
-                intervals.append('()')
-            elif interval.lower == interval.upper:
-                intervals.append('[{}]'.format(repr(interval.lower)))
+        string = []
+        for interval in self._intervals:
+            if interval.lower == interval.upper:
+                string.append("[{}]".format(repr(interval.lower)))
             else:
-                intervals.append(
-                    '{}{},{}{}'.format(
-                        '[' if interval.left == Bound.CLOSED else '(',
+                string.append(
+                    "{}{},{}{}".format(
+                        "[" if interval.left == Bound.CLOSED else "(",
                         repr(interval.lower),
                         repr(interval.upper),
-                        ']' if interval.right == Bound.CLOSED else ')',
+                        "]" if interval.right == Bound.CLOSED else ")",
                     )
                 )
-        return ' | '.join(intervals)
\ No newline at end of file
+        return " | ".join(string)
